package zero1hd.rhythmbullet.graphics.meshes;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.VertexAttribute;
import com.badlogic.gdx.graphics.VertexAttributes.Usage;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;
import com.badlogic.gdx.utils.Disposable;

public class TitleBarMesh implements Disposable {
	private Mesh mesh;
	private OrthographicCamera cam;
	private ShaderProgram shader;
	private Color color;
	private int height;
	
	public final int POSITION_VARS = 2;
	public final int COLOR_VARS = 1; //Packed color data
	public final int TOTAL_VARS = POSITION_VARS + COLOR_VARS;
	public final int MAX_POLY = 1;
	public final int MAX_VERTS = 4 * MAX_POLY;

	private float[] verts = new float[MAX_VERTS * TOTAL_VARS];


	public TitleBarMesh() {
		shader = new ShaderProgram(Gdx.files.internal("shaders/mesh.vsh"), Gdx.files.internal("shaders/mesh.fsh"));
		mesh = new Mesh(true, MAX_VERTS, 0, new VertexAttribute(Usage.Position, POSITION_VARS, "a_position"),
				//still expects 4 color components
				new VertexAttribute(Usage.ColorPacked, 4, "a_color"));
		
		color = new Color(Color.WHITE);
	}
	
	public void setCam(OrthographicCamera cam) {
		this.cam = cam;
	}
	
	public void setColor(Color color) {
		this.color = color;
	}
	
	public void setHeight(int height) {
		this.height = height;
	}
	
	public int getHeight() {
		return height;
	}
	
	public Color getColor() {
		return color;
	}
	
	private int idx = 0;

	public void draw() {
		if (idx == verts.length) {
			flush();
		}
		
		float c = color.toFloatBits();
		
		verts[idx++] = 0;
		verts[idx++] = height;
		verts[idx++] = c;
		
		verts[idx++] = Gdx.graphics.getWidth();
		verts[idx++] = Gdx.graphics.getHeight();
		verts[idx++] = c;

		verts[idx++] = Gdx.graphics.getWidth();
		verts[idx++] = Gdx.graphics.getHeight() - height;
		verts[idx++] = c;

		verts[idx++] = 0;
		verts[idx++] = 0;
		verts[idx++] = c;
	}

	public void flush() {
		if (idx != 0) {
			mesh.setVertices(verts);
			
			Gdx.gl.glDepthMask(false);
			Gdx.gl.glEnable(GL20.GL_BLEND);
			Gdx.gl.glBlendFunc(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);

			int vertCount = idx/TOTAL_VARS;
			
			cam.setToOrtho(false, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			
			shader.begin();
			shader.setUniformMatrix("u_projTrans", cam.combined);
			mesh.render(shader, GL20.GL_TRIANGLE_FAN, 0, vertCount);
			shader.end();
			
			Gdx.gl.glDepthMask(true);
			
			idx = 0;
		}
	}
	
	@Override
	public void dispose() {
		mesh.dispose();
		shader.dispose();
	}
}
